@@ -94,6 +94,36 @@ class InterpolatableTest(unittest.TestCase):
         otf_paths = self.get_file_list(self.tempdir, suffix)
         self.assertIsNone(interpolatable_main(otf_paths))
 
+    @pytest.mark.skipif(
+        sys.version_info[:2] == (3, 8), reason="Fails on Python 3.8 for unknown reasons"
+    )
+    def test_interpolatable_cff2(self):
+        suffix = ".otf"
+        ttx_dir = self.get_test_input("variable_ttx_interpolatable_cff2")
+        ttx_path = os.path.abspath(os.path.join(ttx_dir, "interpolatable-test.ttx"))
+
+        self.temp_dir()
+        self.compile_font(ttx_path, suffix, self.tempdir)
+
+        otf_path = self.get_file_list(self.tempdir, suffix)[0]
+
+        problems = interpolatable_main([otf_path])
+        print(problems)
+        self.assertEqual(
+            problems["uni0408"],
+            [
+                {
+                    "type": "underweight",
+                    "contour": 0,
+                    "master_1": "'wght=200.0 opsz=20.0'",
+                    "master_2": "'wght=200.0 opsz=60.0'",
+                    "master_1_idx": 2,
+                    "master_2_idx": 3,
+                    "tolerance": pytest.approx(0.9184032411892079),
+                },
+            ],
+        )
+
     def test_interpolatable_ufo(self):
         ttx_dir = self.get_test_input("master_ufo")
         ufo_paths = self.get_file_list(ttx_dir, ".ufo", "TestFamily2-")
@@ -258,8 +288,6 @@ class InterpolatableTest(unittest.TestCase):
         input_path = self.get_test_input(
             "..", "..", "ttLib", "data", "varc-ac00-ac01.ttf"
         )
-        # This particular test font which was generated by machine-learning
-        # exhibits an "error" in one of the masters; it's a false-positive.
         # Just make sure the code runs.
         interpolatable_main((input_path,))
 
